#include "../rospell/utils.h"

static bool get_line(char *ptr) {
	ASSERT(ptr != 0);
	while (*ptr != '\0' && *ptr != '\n')
		ptr++;
	
	if (*ptr == '\0')
		return false;
	*ptr = '\0';
	return true;
}

static void usage() {
	printf("Usage: rohtml file\n");
}

static void process(const char *ptr) {
	static bool incode = false;
	
	if (incode && *ptr != ' ') {
		printf("</pre></blockquote>\n");
		incode = false;
	}

	if (!incode && *ptr == ' ') {
		printf("<blockquote><pre>\n%s\n", ptr);
		incode = true;
		return;
	}
	
	if (*ptr == '\0') { // insert <br><br>
		printf("<br><br>\n");
		return;
	}

	// check for section
	if (*ptr == '=' && *(ptr + 1) != '=') {
		// look for the second = sign
		const char *ptr1 = ptr + 1;
		while (*ptr1 != '=' && *ptr1 != '\0')
			ptr1++;
		if (*ptr1 == '=') {
			unsigned len = (unsigned) ptr1 - (unsigned) (ptr + 1);
			char *ptrprint = new char[len + 1];
			strncpy(ptrprint, ptr + 1, len);
			ptrprint[len] = '\0';
			printf("<h1>%s</h1>\n", ptrprint);
			delete[] ptrprint;
//			process(ptr1 + 1);
			return;
		}
	}

	// check for sub-section
	if (*ptr == '=' && *(ptr + 1) == '=' && *(ptr + 2) != '=') {
		// look for the second = sign
		const char *ptr1 = ptr + 2;
		while (*ptr1 != '=' && *ptr1 != '\0')
			ptr1++;
		if (*ptr1 == '=' && *(ptr1 + 1) == '=') {
			unsigned len = (unsigned) ptr1 - (unsigned) (ptr + 2);
			char *ptrprint = new char[len + 1];
			strncpy(ptrprint, ptr + 2, len);
			ptrprint[len] = '\0';
			printf("<h2>%s</h2>\n", ptrprint);
			delete[] ptrprint;
			process(ptr1 + 2);
			return;
		}
	}

	printf("%s\n", ptr);
}

int main(int argc, char **argv) {
	if (argc != 2) {
		usage();
		return 1;
	}
	
	char *strin = UTILS::read_file_malloc(argv[1]);
	if (strin == 0) {
		printf("Error: cannot open file %s\n", argv[1]);
		usage();
		return 1;
	}
	
	char *ptr = strin;
	bool notdone = true;
	do {
		notdone = get_line(ptr);
		process(ptr);
		ptr += strlen(ptr) + 1;
	} while (notdone);
	
	delete[] strin;
	
}
